"use strict";
/**
 * Copyright 2022 IBM Corp. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var configurationhandler_1 = __importDefault(require("./configurationhandler"));
var apimanager_1 = require("./utils/apimanager");
var Constants = __importStar(require("./utils/constants"));
var events_handler_1 = __importDefault(require("./utils/events-handler"));
var metering_1 = __importDefault(require("./utils/metering"));
var configurationHandlerInstance = configurationhandler_1.default.getInstance();
var metering = metering_1.default.getInstance();
var overrideServiceUrl;
var AppConfiguration = /** @class */ (function () {
    function AppConfiguration() {
        this.isInitialized = false;
        this.isInitializedContext = false;
        /**
         * Instance of {@link EventEmitter} to listen to live configuration updates.
         *
         * @memberof AppConfiguration
         */
        this.emitter = events_handler_1.default;
        if (!(0, apimanager_1.fetchAvailable)()) {
            throw new Error(''.concat(Constants.APP_CONFIGURATION, 'Either provide your own "fetch" implementation or run in an environment where "fetch" is available.'));
        }
        window.addEventListener('beforeunload', function () {
            metering.sendMetering(true);
        });
    }
    /**
     * Initialize the sdk to connect with your App Configuration service instance.
     *
     * The method throws {@link Error} if any of the params are missing or invalid.
     *
     * @param {string} region - REGION name where the App Configuration service instance is created.
     * @param {string} guid - GUID of the App Configuration service.
     * @param {string} apikey - Client SDK APIKEY of the App Configuration service.
     * @memberof AppConfiguration
     */
    AppConfiguration.prototype.init = function (region, guid, apikey) {
        if (!region || !guid || !apikey) {
            if (!region) {
                throw new Error(''.concat(Constants.APP_CONFIGURATION, 'region is required'));
            }
            else if (!guid) {
                throw new Error(''.concat(Constants.APP_CONFIGURATION, 'guid is required'));
            }
            else {
                throw new Error(''.concat(Constants.APP_CONFIGURATION, 'apikey is required'));
            }
        }
        configurationHandlerInstance.init(region, guid, apikey, overrideServiceUrl);
        this.isInitialized = true;
    };
    /**
     * Sets the context of the SDK.
     *
     * throws {@link Error} if {@link init} is not performed before calling this method.
     *
     * throws {@link Error} if `collectionId` is not passed or invalid.
     *
     * throws {@link Error} if `environmentId` is not passed or invalid.
     *
     * @param {string} collectionId - Id of the collection created in App Configuration service instance.
     * @param {string} environmentId - Id of the environment created in App Configuration service instance.
     * @return {*}  \{Promise<void>\}
     * @memberof AppConfiguration
     */
    AppConfiguration.prototype.setContext = function (collectionId, environmentId) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.isInitialized) {
                            throw new Error(''.concat(Constants.APP_CONFIGURATION, 'init should be called first'));
                        }
                        if (!collectionId) {
                            throw new Error(''.concat(Constants.APP_CONFIGURATION, 'collectionId is required'));
                        }
                        if (!environmentId) {
                            throw new Error(''.concat(Constants.APP_CONFIGURATION, 'environmentId is required'));
                        }
                        this.isInitializedContext = true;
                        return [4 /*yield*/, configurationHandlerInstance.setContext(collectionId, environmentId)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Returns the {@link Feature} object with the details of the feature flag specified by the `featureId`.
     *
     * throws {@link Error} if the featureId does not exists or invalid.
     *
     * throws {@link Error} if called before SDK is initialised.
     *
     * @param {string} featureId - The Feature flag Id.
     * @return {*}  {@link Feature} object.
     * @memberof AppConfiguration
     */
    AppConfiguration.prototype.getFeature = function (featureId) {
        if (this.isInitialized && this.isInitializedContext) {
            return configurationHandlerInstance.getFeature(featureId);
        }
        throw new Error(''.concat(Constants.APP_CONFIGURATION, 'getFeature: SDK not initialised.'));
    };
    /**
     * Returns the object of type \{ [x: string]: Feature \} containing details of all the feature flags associated with the `collectionId`.
     *
     * @return {*}  \{ [x: string]: {@link Feature }\}
     * @memberof AppConfiguration
     */
    AppConfiguration.prototype.getFeatures = function () {
        if (this.isInitialized && this.isInitializedContext) {
            return configurationHandlerInstance.getFeatures();
        }
        throw new Error(''.concat(Constants.APP_CONFIGURATION, 'getFeatures: SDK not initialised.'));
    };
    /**
     * Returns the {@link Property} object with the details of the property specified by the `propertyId`.
     *
     * throws {@link Error} if the propertyId does not exists or invalid.
     *
     * throws {@link Error} if called before SDK is initialised.
     *
     * @param {string} propertyId
     * @return {*}  \{Property\}
     * @memberof AppConfiguration
     */
    AppConfiguration.prototype.getProperty = function (propertyId) {
        if (this.isInitialized && this.isInitializedContext) {
            return configurationHandlerInstance.getProperty(propertyId);
        }
        throw new Error(''.concat(Constants.APP_CONFIGURATION, 'getProperty: SDK not initialised.'));
    };
    /**
     * Returns the object of type \{ [x: string]: Property \} containing details of all the properties associated with the `collectionId`.
     *
     * @return {*}  \{ [x: string]: {@link Property } \}
     * @memberof AppConfiguration
     */
    AppConfiguration.prototype.getProperties = function () {
        if (this.isInitialized && this.isInitializedContext) {
            return configurationHandlerInstance.getProperties();
        }
        throw new Error(''.concat(Constants.APP_CONFIGURATION, 'getProperties: SDK not initialised.'));
    };
    /**
     * @internal
     *
     * Override the default App Configuration URL. This method should be invoked before the SDK initialization.
     * ```js
     * // Example
     * AppConfiguration.overrideServiceUrl('https://testurl.com');
     * ```
     *  NOTE: To be used for development purposes only.
     * @param url - The base url
     */
    AppConfiguration.overrideServiceUrl = function (url) {
        if (url) {
            overrideServiceUrl = url;
            return;
        }
        throw new Error(''.concat(Constants.APP_CONFIGURATION, 'Unexpected url passed to overrideServiceUrl()'));
    };
    /**
     * Get the instance of {@link AppConfiguration}.
     *
     * @static
     * @return {*}
     * @memberof AppConfiguration
     */
    AppConfiguration.getInstance = function () {
        if (!AppConfiguration.instance) {
            AppConfiguration.instance = new AppConfiguration();
        }
        return AppConfiguration.instance;
    };
    AppConfiguration.REGION_US_SOUTH = 'us-south';
    AppConfiguration.REGION_EU_GB = 'eu-gb';
    AppConfiguration.REGION_AU_SYD = 'au-syd';
    AppConfiguration.REGION_US_EAST = 'us-east';
    AppConfiguration.REGION_EU_DE = 'eu-de';
    AppConfiguration.REGION_CA_TOR = 'ca-tor';
    AppConfiguration.REGION_JP_TOK = 'jp-tok';
    AppConfiguration.REGION_JP_OSA = 'jp-osa';
    return AppConfiguration;
}());
exports.default = AppConfiguration;
